@@ -126,7 +126,7 @@ up_client_get_device (UpClient *client, const gchar *object_path)
  * You must have called up_client_enumerate_devices_sync() before calling this
  * function.
  *
- * Return value: (transfer full): an array of #UpDevice objects, free with g_ptr_array_unref()
+ * Return value: (element-type UpDevice) (transfer full): an array of #UpDevice objects, free with g_ptr_array_unref()
  *
  * Since: 0.9.0
  **/
@@ -277,7 +277,7 @@ up_client_about_to_sleep_sync (UpClient *client,
 	g_return_val_if_fail (client->priv->proxy != NULL, FALSE);
 
 	ret = dbus_g_proxy_call (client->priv->proxy, "AboutToSleep", &error_local,
-				 G_TYPE_STRING, sleep_kind,
+				 G_TYPE_STRING, up_sleep_kind_to_string (sleep_kind),
 				 G_TYPE_INVALID,
 				 G_TYPE_INVALID);
 	if (!ret) {
@@ -360,7 +360,7 @@ up_client_get_properties_sync (UpClient *client, GCancellable *cancellable, GErr
 
 	prop_val = g_value_get_boolean (value) && allowed;
 	if (prop_val != client->priv->can_suspend) {
-		client->priv->can_suspend = ret;
+		client->priv->can_suspend = prop_val;
 		g_object_notify (G_OBJECT(client), "can-suspend");
 	}
 
@@ -376,7 +376,7 @@ up_client_get_properties_sync (UpClient *client, GCancellable *cancellable, GErr
 
 	prop_val = g_value_get_boolean (value) && allowed;
 	if (prop_val != client->priv->can_hibernate) {
-		client->priv->can_hibernate = ret;
+		client->priv->can_hibernate = prop_val;
 		g_object_notify (G_OBJECT(client), "can-hibernate");
 	}
 #endif /* ENABLE_DEPRECATED */
@@ -388,7 +388,7 @@ up_client_get_properties_sync (UpClient *client, GCancellable *cancellable, GErr
 	}
 	prop_val = g_value_get_boolean (value);
 	if (prop_val != client->priv->lid_is_closed) {
-		client->priv->lid_is_closed = ret;
+		client->priv->lid_is_closed = prop_val;
 		g_object_notify (G_OBJECT(client), "lid-is-closed");
 	}
 
@@ -399,7 +399,7 @@ up_client_get_properties_sync (UpClient *client, GCancellable *cancellable, GErr
 	}
 	prop_val = g_value_get_boolean (value);
 	if (prop_val != client->priv->on_battery) {
-		client->priv->on_battery = ret;
+		client->priv->on_battery = prop_val;
 		g_object_notify (G_OBJECT(client), "on-battery");
 	}
 
@@ -410,7 +410,7 @@ up_client_get_properties_sync (UpClient *client, GCancellable *cancellable, GErr
 	}
 	prop_val = g_value_get_boolean (value);
 	if (prop_val != client->priv->on_low_battery) {
-		client->priv->on_low_battery = ret;
+		client->priv->on_low_battery = prop_val;
 		g_object_notify (G_OBJECT(client), "on-low-battery");
 	}
 
@@ -421,7 +421,7 @@ up_client_get_properties_sync (UpClient *client, GCancellable *cancellable, GErr
 	}
 	prop_val = g_value_get_boolean (value);
 	if (prop_val != client->priv->lid_is_present) {
-		client->priv->lid_is_present = ret;
+		client->priv->lid_is_present = prop_val;
 		g_object_notify (G_OBJECT(client), "lid-is-present");
 	}
 
@@ -431,8 +431,8 @@ up_client_get_properties_sync (UpClient *client, GCancellable *cancellable, GErr
 		goto out;
 	}
 	prop_val = g_value_get_boolean (value);
-	if (ret != client->priv->is_docked) {
-		client->priv->is_docked = ret;
+	if (prop_val != client->priv->is_docked) {
+		client->priv->is_docked = prop_val;
 		g_object_notify (G_OBJECT(client), "is-docked");
 	}
 
@@ -443,7 +443,7 @@ up_client_get_properties_sync (UpClient *client, GCancellable *cancellable, GErr
 	}
 	prop_val = g_value_get_boolean (value);
 	if (prop_val != client->priv->lid_force_sleep) {
-		client->priv->lid_force_sleep = ret;
+		client->priv->lid_force_sleep = prop_val;
 		g_object_notify (G_OBJECT(client), "lid-force-sleep");
 	}
 
