@@ -29,6 +29,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <errno.h>
 
 #include "hidpp-device.h"
 
@@ -37,15 +38,10 @@
 
 #define HIDPP_RECEIVER_ADDRESS					0xff
 
-#define HIDPP_RESPONSE_SHORT_LENGTH				7
-#define HIDPP_RESPONSE_LONG_LENGTH				20
-
-#define HIDPP_HEADER_REQUEST					0x10
-#define HIDPP_HEADER_RESPONSE					0x11
-
 /* HID++ 1.0 */
 #define HIDPP_READ_SHORT_REGISTER				0x81
 #define HIDPP_READ_SHORT_REGISTER_BATTERY			0x0d
+#define HIDPP_READ_SHORT_REGISTER_BATTERY_APPROX		0x07
 
 #define HIDPP_READ_LONG_REGISTER				0x83
 #define HIDPP_READ_LONG_REGISTER_DEVICE_TYPE			11
@@ -60,7 +56,22 @@
 #define HIDPP_READ_LONG_REGISTER_DEVICE_TYPE_GAMEPAD		0xb
 #define HIDPP_READ_LONG_REGISTER_DEVICE_TYPE_JOYSTICK		0xc
 
-#define HIDPP_ERR_INVALID_SUBID					0x8f
+#define HIDPP_ERROR_MESSAGE					0x8f
+
+/* HID++ 1.0 error codes */
+#define HIDPP10_ERROR_CODE_SUCCESS				0x00
+#define HIDPP10_ERROR_CODE_INVALID_SUBID			0x01
+#define HIDPP10_ERROR_CODE_INVALID_ADDRESS			0x02
+#define HIDPP10_ERROR_CODE_INVALID_VALUE			0x03
+#define HIDPP10_ERROR_CODE_CONNECT_FAIL				0x04
+#define HIDPP10_ERROR_CODE_TOO_MANY_DEVICES			0x05
+#define HIDPP10_ERROR_CODE_ALREADY_EXISTS			0x06
+#define HIDPP10_ERROR_CODE_BUSY					0x07
+#define HIDPP10_ERROR_CODE_UNKNOWN_DEVICE			0x08
+#define HIDPP10_ERROR_CODE_RESOURCE_ERROR			0x09
+#define HIDPP10_ERROR_CODE_REQUEST_UNAVAILABLE			0x0A
+#define HIDPP10_ERROR_CODE_INVALID_PARAM_VALUE			0x0B
+#define HIDPP10_ERROR_CODE_WRONG_PIN_CODE			0x0C
 
 /* HID++ 2.0 */
 
@@ -108,6 +119,24 @@
 
 #define HIDPP_DEVICE_READ_RESPONSE_TIMEOUT			3000 /* miliseconds */
 
+typedef struct {
+#define HIDPP_MSG_TYPE_SHORT	0x10
+#define HIDPP_MSG_TYPE_LONG	0x11
+#define HIDPP_MSG_LENGTH(msg) ((msg)->type == HIDPP_MSG_TYPE_SHORT ? 7 : 20)
+	guchar			 type;
+	guchar			 device_idx;
+	guchar			 feature_idx;
+	guchar			 function_idx; /* funcId:software_id */
+	union {
+		struct {
+			guchar	 params[3];
+		} s; /* short */
+		struct {
+			guchar	 params[16];
+		} l; /* long */
+	};
+} HidppMessage;
+
 struct HidppDevicePrivate
 {
 	gboolean		 enable_debug;
@@ -120,8 +149,12 @@ struct HidppDevicePrivate
 	guint			 device_idx;
 	guint			 version;
 	HidppDeviceBattStatus	 batt_status;
+	gboolean		 batt_is_approx;
 	HidppDeviceKind		 kind;
 	int			 fd;
+	gboolean		 is_present;
+	gchar			*serial;
+	double			 lux;
 };
 
 typedef struct {
@@ -133,6 +166,24 @@ typedef struct {
 G_DEFINE_TYPE (HidppDevice, hidpp_device, G_TYPE_OBJECT)
 #define HIDPP_DEVICE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), HIDPP_TYPE_DEVICE, HidppDevicePrivate))
 
+/**
+ * hidpp_is_error:
+ *
+ * Checks whether a message is a protocol-level error.
+ */
+static gboolean
+hidpp_is_error(HidppMessage *msg, guchar *error)
+{
+	if (msg->type == HIDPP_MSG_TYPE_SHORT &&
+		msg->feature_idx == HIDPP_ERROR_MESSAGE) {
+		if (error)
+			*error = msg->s.params[1];
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 /**
  * hidpp_device_map_print:
  **/
@@ -197,68 +248,55 @@ hidpp_device_map_get_by_idx (HidppDevice *device, gint idx)
  * Pretty print the send/recieve buffer.
  **/
 static void
-hidpp_device_print_buffer (HidppDevice *device, const guint8 *buffer)
+hidpp_device_print_buffer (HidppDevice *device, const HidppMessage *msg)
 {
-	guint i;
+	guint i, mlen;
 	const HidppDeviceMap *map;
 
 	if (!device->priv->enable_debug)
 		return;
-	for (i = 0; i < HIDPP_RESPONSE_LONG_LENGTH; i++)
-		g_print ("%02x ", buffer[i]);
+
+	mlen = HIDPP_MSG_LENGTH(msg);
+	for (i = 0; i < mlen; i++)
+		g_print ("%02x ", ((const guchar*) msg)[i]);
 	g_print ("\n");
 
-	/* direction */
-	if (buffer[0] == HIDPP_HEADER_REQUEST)
-		g_print ("REQUEST\n");
-	else if (buffer[0] == HIDPP_HEADER_RESPONSE)
-		g_print ("RESPONSE\n");
+	/* direction/type */
+	if (msg->type == HIDPP_MSG_TYPE_SHORT)
+		g_print ("REQUEST/SHORT\n");
+	else if (msg->type == HIDPP_MSG_TYPE_LONG)
+		g_print ("RESPONSE/LONG\n");
 	else
 		g_print ("??\n");
 
 	/* dev index */
-	g_print ("device-idx=%02x ", buffer[1]);
-	if (buffer[1] == HIDPP_RECEIVER_ADDRESS) {
+	g_print ("device-idx=%02x ", msg->device_idx);
+	if (msg->device_idx == HIDPP_RECEIVER_ADDRESS) {
 		g_print ("[Receiver]\n");
-	} else if (device->priv->device_idx == buffer[1]) {
+	} else if (device->priv->device_idx == msg->device_idx) {
 		g_print ("[This Device]\n");
 	} else {
 		g_print ("[Random Device]\n");
 	}
 
 	/* feature index */
-	if (buffer[2] == HIDPP_READ_LONG_REGISTER) {
+	if (msg->feature_idx == HIDPP_READ_LONG_REGISTER) {
 		g_print ("feature-idx=%s [%02x]\n",
-			 "v1(ReadLongRegister)", buffer[2]);
+			 "v1(ReadLongRegister)", msg->feature_idx);
 	} else {
-		map = hidpp_device_map_get_by_idx (device, buffer[2]);
+		map = hidpp_device_map_get_by_idx (device, msg->feature_idx);
 		g_print ("feature-idx=v2(%s) [%02x]\n",
-			 map != NULL ? map->name : "unknown", buffer[2]);
+			 map != NULL ? map->name : "unknown", msg->feature_idx);
 	}
 
-	g_print ("function-id=%01x\n", buffer[3] & 0xf);
-	g_print ("software-id=%01x\n", buffer[3] >> 4);
-	g_print ("param[0]=%02x\n\n", buffer[4]);
+	g_print ("function-id=%01x\n", msg->function_idx & 0xf);
+	g_print ("software-id=%01x\n", msg->function_idx >> 4);
+	g_print ("param[0]=%02x\n\n", msg->s.params[0]);
 }
 
-/**
- * hidpp_device_cmd:
- **/
-static gboolean
-hidpp_device_cmd (HidppDevice	*device,
-		  guint8	 device_idx,
-		  guint8	 feature_idx,
-		  guint8	 function_idx,
-		  guint8	*request_data,
-		  gsize		 request_len,
-		  guint8	*response_data,
-		  gsize		 response_len,
-		  GError	**error)
+static void
+hidpp_discard_messages (HidppDevice	*device)
 {
-	gboolean ret = TRUE;
-	gssize wrote;
-	guint8 buf[HIDPP_RESPONSE_LONG_LENGTH];
-	guint i;
 	HidppDevicePrivate *priv = device->priv;
 	GPollFD poll[] = {
 		{
@@ -266,73 +304,170 @@ hidpp_device_cmd (HidppDevice	*device,
 			.events = G_IO_IN | G_IO_OUT | G_IO_ERR,
 		},
 	};
+	char c;
+	int r;
 
-	/* make the request packet */
-	memset (buf, 0x00, HIDPP_RESPONSE_LONG_LENGTH);
-	buf[0] = HIDPP_HEADER_REQUEST;
-	buf[1] = device_idx;
-	buf[2] = feature_idx;
-	buf[3] = function_idx;
-	for (i = 0; i < request_len; i++)
-		buf[4 + i] = request_data[i];
+	while (g_poll (poll, G_N_ELEMENTS(poll), 0) > 0) {
+		/* kernel discards remainder of packet */
+		r = read (priv->fd, &c, 1);
+		if (r < 0 && errno != EINTR)
+			break;
+	}
+}
+
+static gboolean
+hidpp_device_read_resp (HidppDevice	*device,
+			guchar		 device_index,
+			guchar		 feature_index,
+			guchar		 function_index,
+			HidppMessage	*response,
+			GError	**error);
+
+/**
+ * hidpp_device_cmd:
+ **/
+static gboolean
+hidpp_device_cmd (HidppDevice	*device,
+		  const HidppMessage	*request,
+		  HidppMessage	*response,
+		  GError	**error)
+{
+	gssize wrote;
+	guint msg_len;
+	HidppDevicePrivate *priv = device->priv;
+
+	g_assert (request->type == HIDPP_MSG_TYPE_SHORT ||
+			request->type == HIDPP_MSG_TYPE_LONG);
+
+	hidpp_device_print_buffer (device, request);
+
+	msg_len = HIDPP_MSG_LENGTH(request);
+
+	/* ignore all unrelated queued messages */
+	hidpp_discard_messages(device);
 
 	/* write to the device */
-	hidpp_device_print_buffer (device, buf);
-	wrote = write (priv->fd, buf, 4 + request_len);
-	if ((gsize) wrote != 4 + request_len) {
-		g_set_error (error, 1, 0,
-			     "Unable to write request to device: %" G_GSIZE_FORMAT,
-			     wrote);
-		ret = FALSE;
-		goto out;
+	wrote = write (priv->fd, (const char *)request, msg_len);
+	if ((gsize) wrote != msg_len) {
+		if (wrote < 0) {
+			g_set_error (error, 1, 0,
+					"Failed to write HID++ request: %s",
+					g_strerror (errno));
+		} else {
+			g_set_error (error, 1, 0,
+					"Could not fully write HID++ request, wrote %" G_GSIZE_FORMAT " bytes",
+					wrote);
+		}
+		return FALSE;
 	}
 
+	return hidpp_device_read_resp (device,
+		request->device_idx,
+		request->feature_idx,
+		request->function_idx,
+		response,
+		error);
+}
+
+/**
+ * hidpp_device_read_resp:
+ */
+static gboolean
+hidpp_device_read_resp (HidppDevice	*device,
+			guchar		 device_index,
+			guchar		 feature_index,
+			guchar		 function_index,
+			HidppMessage	*response,
+			GError	**error)
+{
+	HidppDevicePrivate *priv = device->priv;
+	gboolean ret = TRUE;
+	gssize r;
+	GPollFD poll[] = {
+		{
+			.fd = priv->fd,
+			.events = G_IO_IN | G_IO_OUT | G_IO_ERR,
+		},
+	};
+	guint64 begin_time;
+	gint remaining_time;
+	guchar error_code;
+
 	/* read from the device */
-	wrote = g_poll (poll, G_N_ELEMENTS(poll),
-		        HIDPP_DEVICE_READ_RESPONSE_TIMEOUT);
-	if (wrote <= 0) {
-		g_set_error (error, 1, 0,
-			     "Attempt to read response from device timed out: %" G_GSIZE_FORMAT,
-			     wrote);
-		ret = FALSE;
-		goto out;
-	}
-	memset (buf, 0x00, HIDPP_RESPONSE_LONG_LENGTH);
-	wrote = read (priv->fd, buf, sizeof (buf));
-	if (wrote <= 0) {
-		g_set_error (error, 1, 0,
-			     "Unable to read response from device: %" G_GSIZE_FORMAT,
-			     wrote);
-		ret = FALSE;
-		goto out;
-	}
-
-	/* is device offline */
-	hidpp_device_print_buffer (device, buf);
-	if (buf[0] == HIDPP_HEADER_REQUEST &&
-	    buf[1] == device_idx &&
-	    buf[2] == HIDPP_ERR_INVALID_SUBID &&
-	    buf[3] == 0x00 &&
-	    buf[4] == HIDPP_FEATURE_ROOT_FN_PING) {
-		/* HID++ 1.0 ping reply, so fake success with version 1  */
-		if (priv->version < 2 && (buf[5] == HIDPP_ERROR_CODE_UNKNOWN
-					|| buf[5] == HIDPP_ERROR_CODE_UNSUPPORTED)) {
-			response_data[0] = 1;
+	begin_time = g_get_monotonic_time () / 1000;
+	for (;;) {
+		/* avoid infinite loop when there is no response */
+		remaining_time = HIDPP_DEVICE_READ_RESPONSE_TIMEOUT -
+			(g_get_monotonic_time () / 1000 - begin_time);
+		if (remaining_time <= 0) {
+			g_set_error (error, 1, 0,
+					"timeout while reading response");
+			ret = FALSE;
 			goto out;
 		}
+
+		r = g_poll (poll, G_N_ELEMENTS(poll), remaining_time);
+		if (r < 0) {
+			if (errno == EINTR)
+				continue;
+
+			g_set_error (error, 1, 0,
+					"Failed to read from device: %s",
+					g_strerror (errno));
+			ret = FALSE;
+			goto out;
+		} else if (r == 0) {
+			g_set_error (error, 1, 0,
+					"Attempt to read response from device timed out");
+			ret = FALSE;
+			goto out;
+		}
+
+		r = read (priv->fd, response, sizeof (*response));
+		if (r <= 0) {
+			if (r == -1 && errno == EINTR)
+				continue;
+
+			g_set_error (error, 1, 0,
+					"Unable to read response from device: %s",
+					g_strerror (errno));
+			ret = FALSE;
+			goto out;
+		}
+
+		hidpp_device_print_buffer (device, response);
+
+		/* validate response */
+		if (response->type != HIDPP_MSG_TYPE_SHORT &&
+			response->type != HIDPP_MSG_TYPE_LONG) {
+			/* ignore key presses, mouse motions, etc. */
+			continue;
+		}
+
+		/* not our device */
+		if (response->device_idx != device_index) {
+			continue;
+		}
+
+		/* yep, this is our request */
+		if (response->feature_idx == feature_index &&
+			response->function_idx == function_index) {
+			break;
+		}
+
+		/* recognize HID++ 1.0 errors */
+		if (hidpp_is_error(response, &error_code) &&
+			response->function_idx == feature_index &&
+			response->s.params[0] == function_index) {
+			g_set_error (error, 1, 0,
+				"Unable to satisfy request, HID++ error %02x", error_code);
+			ret = FALSE;
+			goto out;
+		}
+
+		/* not our message, ignore it and try again */
 	}
-	if ((buf[0] != HIDPP_HEADER_REQUEST && buf[0] != HIDPP_HEADER_RESPONSE) ||
-	    buf[1] != device_idx ||
-	    buf[2] != feature_idx ||
-	    buf[3] != function_idx) {
-		g_set_error (error, 1, 0,
-			     "invalid response from device: %" G_GSIZE_FORMAT,
-			     wrote);
-		ret = FALSE;
-		goto out;
-	}
-	for (i = 0; i < response_len; i++)
-		response_data[i] = buf[4 + i];
+
 out:
 	return ret;
 }
@@ -350,21 +485,21 @@ hidpp_device_map_add (HidppDevice *device,
 {
 	gboolean ret;
 	GError *error = NULL;
-	guint8 buf[3];
+	HidppMessage msg;
 	HidppDeviceMap *map;
 	HidppDevicePrivate *priv = device->priv;
 
-	buf[0] = feature >> 8;
-	buf[1] = feature;
-	buf[2] = 0x00;
+	msg.type = HIDPP_MSG_TYPE_SHORT;
+	msg.device_idx = priv->device_idx;
+	msg.feature_idx = HIDPP_FEATURE_ROOT_INDEX;
+	msg.function_idx = HIDPP_FEATURE_ROOT_FN_GET_FEATURE;
+	msg.s.params[0] = feature >> 8;
+	msg.s.params[1] = feature;
+	msg.s.params[2] = 0x00;
 
 	g_debug ("Getting idx for feature %s [%02x]", name, feature);
 	ret = hidpp_device_cmd (device,
-				priv->device_idx,
-				HIDPP_FEATURE_ROOT_INDEX,
-				HIDPP_FEATURE_ROOT_FN_GET_FEATURE,
-				buf, sizeof (buf),
-				buf, sizeof (buf),
+				&msg, &msg,
 				&error);
 	if (!ret) {
 		g_warning ("Failed to get feature idx: %s", error->message);
@@ -373,7 +508,7 @@ hidpp_device_map_add (HidppDevice *device,
 	}
 
 	/* zero index */
-	if (buf[0] == 0x00) {
+	if (msg.s.params[0] == 0x00) {
 		ret = FALSE;
 		g_debug ("Feature not found");
 		goto out;
@@ -381,7 +516,7 @@ hidpp_device_map_add (HidppDevice *device,
 
 	/* add to map */
 	map = g_new0 (HidppDeviceMap, 1);
-	map->idx = buf[0];
+	map->idx = msg.s.params[0];
 	map->feature = feature;
 	map->name = g_strdup (name);
 	g_ptr_array_add (priv->feature_index, map);
@@ -441,6 +576,37 @@ hidpp_device_get_kind (HidppDevice *device)
 	return device->priv->kind;
 }
 
+/**
+ * hidpp_device_get_serial:
+ **/
+const gchar *
+hidpp_device_get_serial (HidppDevice *device)
+{
+	g_return_val_if_fail (HIDPP_IS_DEVICE (device), NULL);
+	return device->priv->serial;
+}
+
+/**
+ * hidpp_device_is_reachable:
+ **/
+gboolean
+hidpp_device_is_reachable (HidppDevice *device)
+{
+	g_return_val_if_fail (HIDPP_IS_DEVICE (device), FALSE);
+	return device->priv->is_present;
+}
+
+/**
+ * hidpp_device_get_luminosity:
+ * Determine the luminosity of the device in lux or negative if unknown.
+ */
+double
+hidpp_device_get_luminosity (HidppDevice *device)
+{
+	g_return_val_if_fail (HIDPP_IS_DEVICE (device), -1);
+	return device->priv->lux;
+}
+
 /**
  * hidpp_device_set_hidraw_device:
  **/
@@ -484,11 +650,10 @@ hidpp_device_refresh (HidppDevice *device,
 {
 	const HidppDeviceMap *map;
 	gboolean ret = TRUE;
-	GString *name = NULL;
-	guint8 buf[HIDPP_RESPONSE_LONG_LENGTH];
-	guint i;
+	HidppMessage msg = { };
 	guint len;
 	HidppDevicePrivate *priv = device->priv;
+	guchar error_code = 0;
 
 	g_return_val_if_fail (HIDPP_IS_DEVICE (device), FALSE);
 
@@ -508,21 +673,51 @@ hidpp_device_refresh (HidppDevice *device,
 	if ((refresh_flags & HIDPP_REFRESH_FLAGS_VERSION) > 0) {
 		guint version_old = priv->version;
 
-		buf[0] = 0x00;
-		buf[1] = 0x00;
-		buf[2] = HIDPP_PING_DATA;
+		msg.type = HIDPP_MSG_TYPE_SHORT;
+		msg.device_idx = priv->device_idx;
+		msg.feature_idx = HIDPP_FEATURE_ROOT_INDEX;
+		msg.function_idx = HIDPP_FEATURE_ROOT_FN_PING;
+		msg.s.params[0] = 0x00;
+		msg.s.params[1] = 0x00;
+		msg.s.params[2] = HIDPP_PING_DATA;
 		ret = hidpp_device_cmd (device,
-					priv->device_idx,
-					HIDPP_FEATURE_ROOT_INDEX,
-					HIDPP_FEATURE_ROOT_FN_PING,
-					buf, 3,
-					buf, 4,
+					&msg, &msg,
 					error);
+		if (!ret) {
+			if (hidpp_is_error(&msg, &error_code) &&
+				(error_code == HIDPP10_ERROR_CODE_INVALID_SUBID ||
+				/* if a device is unreachable, assume HID++ 1.0.
+				 * By doing so, we are still able to get the
+				 * device type (e.g. mouse or keyboard) at
+				 * enumeration time. */
+				error_code == HIDPP10_ERROR_CODE_RESOURCE_ERROR)) {
+
+				/* assert HID++ 1.0 for the device only if we
+				 * are sure (i.e.  when the ping request
+				 * returned INVALID_SUBID) */
+				if (error_code == HIDPP10_ERROR_CODE_INVALID_SUBID) {
+					priv->version = 1;
+					priv->is_present = TRUE;
+				} else {
+					g_debug("Cannot detect version, unreachable device");
+					priv->is_present = FALSE;
+				}
+
+				/* do not execute the error handler at the end
+				 * of this function */
+				memset(&msg, 0, sizeof (msg));
+				g_error_free(*error);
+				*error = NULL;
+				ret = TRUE;
+			}
+		} else {
+			priv->version = msg.s.params[0];
+			priv->is_present = TRUE;
+		}
+
 		if (!ret)
 			goto out;
 
-		priv->version = buf[0];
-
 		if (version_old != priv->version)
 			g_debug("protocol for hid++ device changed from v%d to v%d",
 					version_old, priv->version);
@@ -540,9 +735,9 @@ hidpp_device_refresh (HidppDevice *device,
 //		hidpp_device_map_add (device,
 //				      HIDPP_FEATURE_I_FIRMWARE_INFO,
 //				      "IFirmwareInfo");
-		hidpp_device_map_add (device,
-				HIDPP_FEATURE_GET_DEVICE_NAME_TYPE,
-				"GetDeviceNameType");
+//		hidpp_device_map_add (device,
+//				HIDPP_FEATURE_GET_DEVICE_NAME_TYPE,
+//				"GetDeviceNameType");
 		hidpp_device_map_add (device,
 				HIDPP_FEATURE_BATTERY_LEVEL_STATUS,
 				"BatteryLevelStatus");
@@ -558,20 +753,22 @@ hidpp_device_refresh (HidppDevice *device,
 	/* get device kind */
 	if ((refresh_flags & HIDPP_REFRESH_FLAGS_KIND) > 0) {
 
-		if (priv->version == 1) {
-			buf[0] = 0x20 | (priv->device_idx - 1);
-			buf[1] = 0x00;
-			buf[2] = 0x00;
+		/* the device type can always be queried using HID++ 1.0 on the
+		 * receiver, regardless of the device version. */
+		if (priv->version <= 1 || priv->version == 2) {
+			msg.type = HIDPP_MSG_TYPE_SHORT;
+			msg.device_idx = HIDPP_RECEIVER_ADDRESS;
+			msg.feature_idx = HIDPP_READ_LONG_REGISTER;
+			msg.function_idx = 0xb5;
+			msg.s.params[0] = 0x20 | (priv->device_idx - 1);
+			msg.s.params[1] = 0x00;
+			msg.s.params[2] = 0x00;
 			ret = hidpp_device_cmd (device,
-						HIDPP_RECEIVER_ADDRESS,
-						HIDPP_READ_LONG_REGISTER,
-						0xb5,
-						buf, 3,
-						buf, 8,
+						&msg, &msg,
 						error);
 			if (!ret)
 				goto out;
-			switch (buf[7]) {
+			switch (msg.l.params[7]) {
 			case HIDPP_READ_LONG_REGISTER_DEVICE_TYPE_KEYBOARD:
 			case HIDPP_READ_LONG_REGISTER_DEVICE_TYPE_NUMPAD:
 			case HIDPP_READ_LONG_REGISTER_DEVICE_TYPE_REMOTE_CONTROL:
@@ -592,157 +789,181 @@ hidpp_device_refresh (HidppDevice *device,
 				priv->kind = HIDPP_DEVICE_KIND_UNKNOWN;
 				break;
 			}
-		} else if (priv->version == 2) {
-
-			/* send a BatteryLevelStatus report */
-			map = hidpp_device_map_get_by_feature (device, HIDPP_FEATURE_GET_DEVICE_NAME_TYPE);
-			if (map != NULL) {
-				buf[0] = 0x00;
-				buf[1] = 0x00;
-				buf[2] = 0x00;
-				ret = hidpp_device_cmd (device,
-							priv->device_idx,
-							map->idx,
-							HIDPP_FEATURE_GET_DEVICE_NAME_TYPE_FN_GET_TYPE,
-							buf, 3,
-							buf, 1,
-							error);
-				if (!ret)
-					goto out;
-				switch (buf[0]) {
-				case 0: /* keyboard */
-				case 2: /* numpad */
-					priv->kind = HIDPP_DEVICE_KIND_KEYBOARD;
-					break;
-				case 3: /* mouse */
-				case 4: /* touchpad */
-				case 5: /* trackball */
-					priv->kind = HIDPP_DEVICE_KIND_MOUSE;
-					break;
-				case 1: /* remote-control */
-				case 6: /* presenter */
-				case 7: /* receiver */
-					priv->kind = HIDPP_DEVICE_KIND_UNKNOWN;
-					break;
-				}
-			}
 		}
 	}
 
 	/* get device model string */
 	if ((refresh_flags & HIDPP_REFRESH_FLAGS_MODEL) > 0) {
-		if (priv->version == 1) {
-			buf[0] = 0x40 | (priv->device_idx - 1);
-			buf[1] = 0x00;
-			buf[2] = 0x00;
+		/* the device name can always be queried using HID++ 1.0 on the
+		 * receiver, regardless of the device version. */
+		if (priv->version <= 1 || priv->version == 2) {
+			msg.type = HIDPP_MSG_TYPE_SHORT;
+			msg.device_idx = HIDPP_RECEIVER_ADDRESS;
+			msg.feature_idx = HIDPP_READ_LONG_REGISTER;
+			msg.function_idx = 0xb5;
+			msg.s.params[0] = 0x40 | (priv->device_idx - 1);
+			msg.s.params[1] = 0x00;
+			msg.s.params[2] = 0x00;
 
 			ret = hidpp_device_cmd (device,
-					HIDPP_RECEIVER_ADDRESS,
-					HIDPP_READ_LONG_REGISTER,
-					0xb5,
-					buf, 3,
-					buf, HIDPP_RESPONSE_LONG_LENGTH,
+					&msg, &msg,
 					error);
 			if (!ret)
 				goto out;
 
-			len = buf[1];
-			name = g_string_new ("");
-			g_string_append_len (name, (gchar *) buf+2, len);
-			priv->model = g_strdup (name->str);
-		} else if (priv->version == 2) {
-			buf[0] = 0x00;
-			buf[1] = 0x00;
-			buf[2] = 0x00;
-			map = hidpp_device_map_get_by_feature (device, HIDPP_FEATURE_GET_DEVICE_NAME_TYPE);
-			if (map != NULL) {
-				ret = hidpp_device_cmd (device,
-						priv->device_idx,
-						map->idx,
-						HIDPP_FEATURE_GET_DEVICE_NAME_TYPE_FN_GET_COUNT,
-						buf, 3,
-						buf, 1,
-						error);
-				if (!ret)
-					goto out;
-			}
-			len = buf[0];
-			name = g_string_new ("");
-			for (i = 0; i < len; i +=4 ) {
-				buf[0] = i;
-				buf[1] = 0x00;
-				buf[2] = 0x00;
-				ret = hidpp_device_cmd (device,
-						priv->device_idx,
-						map->idx,
-						HIDPP_FEATURE_GET_DEVICE_NAME_TYPE_FN_GET_NAME,
-						buf, 3,
-						buf, 4,
-						error);
-				if (!ret)
-					goto out;
-				g_string_append_len (name, (gchar *) &buf[0], 4);
-			}
-			priv->model = g_strdup (name->str);
+			len = msg.l.params[1];
+			priv->model = g_strdup_printf ("%.*s", len, msg.l.params + 2);
 		}
 	}
 
+	/* get serial number, this can be queried from the receiver */
+	if ((refresh_flags & HIDPP_REFRESH_FLAGS_SERIAL) > 0) {
+		guint32 *serialp;
+
+		msg.type = HIDPP_MSG_TYPE_SHORT;
+		msg.device_idx = HIDPP_RECEIVER_ADDRESS;
+		msg.feature_idx = HIDPP_READ_LONG_REGISTER;
+		msg.function_idx = 0xb5;
+		msg.s.params[0] = 0x30 | (priv->device_idx - 1);
+		msg.s.params[1] = 0x00;
+		msg.s.params[2] = 0x00;
+
+		ret = hidpp_device_cmd (device,
+				&msg, &msg,
+				error);
+		if (!ret)
+			goto out;
+
+		serialp = (guint32 *) &msg.l.params[1];
+		priv->serial = g_strdup_printf ("%08X", g_ntohl(*serialp));
+	}
+
 	/* get battery status */
 	if ((refresh_flags & HIDPP_REFRESH_FLAGS_BATTERY) > 0) {
 		if (priv->version == 1) {
-			buf[0] = 0x00;
-			buf[1] = 0x00;
-			buf[2] = 0x00;
-			ret = hidpp_device_cmd (device,
-						priv->device_idx,
-						HIDPP_READ_SHORT_REGISTER,
-						HIDPP_READ_SHORT_REGISTER_BATTERY,
-						buf, 3,
-						buf, 1,
-						error);
+			if (!priv->batt_is_approx) {
+				msg.type = HIDPP_MSG_TYPE_SHORT;
+				msg.device_idx = priv->device_idx;
+				msg.feature_idx = HIDPP_READ_SHORT_REGISTER;
+				msg.function_idx = HIDPP_READ_SHORT_REGISTER_BATTERY;
+				memset(msg.s.params, 0, sizeof(msg.s.params));
+				ret = hidpp_device_cmd (device,
+							&msg, &msg,
+							error);
+				if (!ret && hidpp_is_error(&msg, &error_code) &&
+					error_code == HIDPP10_ERROR_CODE_INVALID_ADDRESS) {
+					g_error_free(*error);
+					*error = NULL;
+					priv->batt_is_approx = TRUE;
+				}
+			}
+
+			if (priv->batt_is_approx) {
+				msg.type = HIDPP_MSG_TYPE_SHORT;
+				msg.device_idx = priv->device_idx;
+				msg.feature_idx = HIDPP_READ_SHORT_REGISTER;
+				msg.function_idx = HIDPP_READ_SHORT_REGISTER_BATTERY_APPROX;
+				memset(msg.s.params, 0, sizeof(msg.s.params));
+
+				ret = hidpp_device_cmd (device,
+							&msg, &msg,
+							error);
+			}
 			if (!ret)
 				goto out;
-			priv->batt_percentage = buf[0];
-			priv->batt_status = HIDPP_DEVICE_BATT_STATUS_DISCHARGING;
+			if (msg.function_idx == HIDPP_READ_SHORT_REGISTER_BATTERY) {
+				priv->batt_percentage = msg.s.params[0];
+				priv->batt_status = HIDPP_DEVICE_BATT_STATUS_DISCHARGING;
+			} else {
+				/* approximate battery levels */
+				switch (msg.s.params[0]) {
+				case 1: /* 0 - 10 */
+					priv->batt_percentage = 5;
+					break;
+				case 3: /* 11 - 30 */
+					priv->batt_percentage = 20;
+					break;
+				case 5: /* 31 - 80 */
+					priv->batt_percentage = 55;
+					break;
+				case 7: /* 81 - 100 */
+					priv->batt_percentage = 90;
+					break;
+				default:
+					g_debug("Unknown battery percentage: %i", priv->batt_percentage);
+					break;
+				}
+				switch (msg.s.params[1]) {
+				case 0x00:
+					priv->batt_status = HIDPP_DEVICE_BATT_STATUS_DISCHARGING;
+					break;
+				case 0x22:
+				case 0x26: /* for notification, probably N/A for reg read */
+					priv->batt_status = HIDPP_DEVICE_BATT_STATUS_CHARGED;
+					break;
+				case 0x25:
+					priv->batt_status = HIDPP_DEVICE_BATT_STATUS_CHARGING;
+					break;
+				default:
+					g_debug("Unknown battery status: 0x%02x", priv->batt_status);
+					break;
+				}
+			}
 		} else if (priv->version == 2) {
 
 			/* sent a SetLightMeasure report */
 			map = hidpp_device_map_get_by_feature (device, HIDPP_FEATURE_SOLAR_DASHBOARD);
 			if (map != NULL) {
-				buf[0] = 0x01; /* Max number of reports: number of report sent after function call */
-				buf[1] = 0x01; /* Report period: time between reports, in seconds */
+				msg.type = HIDPP_MSG_TYPE_SHORT;
+				msg.device_idx = priv->device_idx;
+				msg.feature_idx = map->idx;
+				msg.function_idx = HIDPP_FEATURE_SOLAR_DASHBOARD_FN_SET_LIGHT_MEASURE;
+				msg.s.params[0] = 0x01; /* Max number of reports: number of report sent after function call */
+				msg.s.params[1] = 0x01; /* Report period: time between reports, in seconds */
 				ret = hidpp_device_cmd (device,
-							priv->device_idx,
-							map->idx,
-							HIDPP_FEATURE_SOLAR_DASHBOARD_FN_SET_LIGHT_MEASURE,
-							buf, 2,
-							buf, 3,
+							&msg, &msg,
 							error);
 				if (!ret)
 					goto out;
-				priv->batt_percentage = buf[0];
-				priv->batt_status = HIDPP_DEVICE_BATT_STATUS_DISCHARGING;
+
+				/* assume a BattLightMeasureEvent after previous command */
+				ret = hidpp_device_read_resp (device,
+							priv->device_idx,
+							map->idx,
+							HIDPP_FEATURE_SOLAR_DASHBOARD_BE_BATTERY_LEVEL_STATUS,
+							&msg,
+							error);
+				if (!ret)
+					goto out;
+
+				priv->batt_percentage = msg.l.params[0];
+				priv->lux = (msg.l.params[1] << 8) | msg.l.params[2];
+				if (priv->lux > 200) {
+					priv->batt_status = HIDPP_DEVICE_BATT_STATUS_CHARGING;
+				} else {
+					priv->batt_status = HIDPP_DEVICE_BATT_STATUS_DISCHARGING;
+				}
 			}
 
 			/* send a BatteryLevelStatus report */
 			map = hidpp_device_map_get_by_feature (device, HIDPP_FEATURE_BATTERY_LEVEL_STATUS);
 			if (map != NULL) {
-				buf[0] = 0x00;
-				buf[1] = 0x00;
-				buf[2] = 0x00;
+				msg.type = HIDPP_MSG_TYPE_SHORT;
+				msg.device_idx = priv->device_idx;
+				msg.feature_idx = map->idx;
+				msg.function_idx = HIDPP_FEATURE_BATTERY_LEVEL_STATUS_FN_GET_STATUS;
+				msg.s.params[0] = 0x00;
+				msg.s.params[1] = 0x00;
+				msg.s.params[2] = 0x00;
 				ret = hidpp_device_cmd (device,
-							priv->device_idx,
-							map->idx,
-							HIDPP_FEATURE_BATTERY_LEVEL_STATUS_FN_GET_STATUS,
-							buf, 3,
-							buf, 3,
+							&msg, &msg,
 							error);
 				if (!ret)
 					goto out;
 
 				/* convert the HID++ v2 status into something
 				 * we can set on the device */
-				switch (buf[2]) {
+				switch (msg.s.params[2]) {
 				case 0: /* discharging */
 					priv->batt_status = HIDPP_DEVICE_BATT_STATUS_DISCHARGING;
 					break;
@@ -752,20 +973,44 @@ hidpp_device_refresh (HidppDevice *device,
 					priv->batt_status = HIDPP_DEVICE_BATT_STATUS_CHARGING;
 					break;
 				case 3: /* charging complete */
+					priv->batt_percentage = 100;
 					priv->batt_status = HIDPP_DEVICE_BATT_STATUS_CHARGED;
 					break;
 				default:
 					break;
 				}
-				priv->batt_percentage = buf[0];
+
+				/* do not overwrite battery status with 0 (unknown) */
+				if (msg.s.params[0] != 0)
+					priv->batt_percentage = msg.s.params[0];
+
 				g_debug ("level=%i%%, next-level=%i%%, battery-status=%i",
-					 buf[0], buf[1], buf[2]);
+					 msg.s.params[0], msg.s.params[1], msg.s.params[2]);
 			}
 		}
 	}
+
+	/* when no error occured for the requests done by the following refresh
+	 * flags, assume the device present. Note that the is_present flag is
+	 * always set when using HIDPP_REFRESH_FLAGS_VERSION */
+	if (priv->version > 0 && refresh_flags &
+			(HIDPP_REFRESH_FLAGS_MODEL |
+			 HIDPP_REFRESH_FLAGS_BATTERY)) {
+		priv->is_present = TRUE;
+	}
 out:
-	if (name != NULL)
-		g_string_free (name, TRUE);
+	/* do not spam when device is unreachable */
+	if (hidpp_is_error(&msg, &error_code) &&
+			(error_code == HIDPP10_ERROR_CODE_RESOURCE_ERROR)) {
+		g_debug("HID++ error: %s", (*error)->message);
+		g_error_free(*error);
+		*error = NULL;
+		/* the device is unreachable but paired, consider the refresh
+		 * successful. Use is_present to determine if battery
+		 * information is actually updated */
+		ret = TRUE;
+		priv->is_present = FALSE;
+	}
 	return ret;
 }
 
@@ -778,10 +1023,14 @@ hidpp_device_init (HidppDevice *device)
 	HidppDeviceMap *map;
 
 	device->priv = HIDPP_DEVICE_GET_PRIVATE (device);
+	device->priv->is_present = FALSE;
 	device->priv->fd = -1;
 	device->priv->feature_index = g_ptr_array_new_with_free_func (g_free);
 	device->priv->batt_status = HIDPP_DEVICE_BATT_STATUS_UNKNOWN;
+	device->priv->batt_is_approx = FALSE;
 	device->priv->kind = HIDPP_DEVICE_KIND_UNKNOWN;
+	device->priv->serial = NULL;
+	device->priv->lux = -1;
 
 	/* add known root */
 	map = g_new0 (HidppDeviceMap, 1);
@@ -816,6 +1065,10 @@ hidpp_device_finalize (GObject *object)
 
 	g_free (device->priv->hidraw_device);
 	g_free (device->priv->model);
+	g_free (device->priv->serial);
+
+	if (device->priv->fd > 0)
+		close (device->priv->fd);
 
 	G_OBJECT_CLASS (hidpp_device_parent_class)->finalize (object);
 }
