@@ -31,11 +31,22 @@
  * This would be implemented on a Linux system using:
  *  g_udev_device_get_sysfs_path (G_UDEV_DEVICE (object))
  *
- * Return value: The native path for the device which is unique, e.g. "/sys/class/power/BAT1"
+ * Return value: Device name for devices of subsystem "power_supply", otherwise
+ * the native path for the device which is unique.
  **/
 const gchar *
 up_native_get_native_path (GObject *object)
 {
+	/* Device names within the same subsystem must be unique. To avoid
+	 * treating the same power supply device on variable buses as different
+	 * only because e. g. the USB or bluetooth tree layout changed, only
+	 * use their name as identification. Also see
+	 * http://bugzilla.kernel.org/show_bug.cgi?id=62041 */
+	if (g_strcmp0 (g_udev_device_get_subsystem (G_UDEV_DEVICE (object)), "power_supply") == 0)
+		return g_udev_device_get_name (G_UDEV_DEVICE (object));
+
+	/* we do not expect other devices than power_supply, but provide this
+	 * fallback for completeness */
 	return g_udev_device_get_sysfs_path (G_UDEV_DEVICE (object));
 }
 
