@@ -788,8 +788,6 @@ up_wakeups_finalize (GObject *object)
 UpWakeups *
 up_wakeups_new (void)
 {
-	UpWakeups *wakeups;
-	wakeups = g_object_new (UP_TYPE_WAKEUPS, NULL);
-	return UP_WAKEUPS (wakeups);
+	return g_object_new (UP_TYPE_WAKEUPS, NULL);
 }
 
